
proc PhotoFrame.methods obj {
 sd.new.method create.parent.image for $obj takes {win pWin} {
  set pImg [$pWin image]

  set x [$self->calculate.offset.to.window $win $pWin x]
  set y [$self->calculate.offset.to.window $win $pWin y]
  set w [winfo width $win]
  set h [winfo height $win]
  set img [image create photo -width $w -height $h]
  set x2 [expr {$x + $w}]
  set y2 [expr {$y + $h}]
 
  $self->make.region.fit.image x y x2 y2 $pImg 
  $img copy $pImg -from $x $y $x2 $y2

  puts "$x $y"
  return $img
 }

 sd.new.method create.tiled.image for $obj takes {win img} {
  set maxX [winfo width $win]
  set maxY [winfo height $win]
  set xIncr [image width $img]
  set yIncr [image height $img]
  set newImg [image create photo -width $maxX -height $maxY]

  for {set y 0} {$y < $maxY} {incr y $yIncr} {
   for {set x 0} {$x < $maxX} {incr x $xIncr} {
    $newImg copy $img -to $x $y [expr {$x + $xIncr}] [expr {$y + $yIncr}]    
   }
  }
  return $newImg
 }

 sd.new.method draw.image for $obj takes {win img} {
  if {"" != [$self img]} {
   image delete [$self img]
  }

  if {"" != [$self id]} {
   $win delete [$self id]
  }
  set x [expr {[image width $img] / 2}]
  set y [expr {[image height $img] / 2}]

  $self id [$win create image $x $y -image $img]
  $self img $img
 }

 sd.new.method event.configure for $obj takes {win w h} {
  if {"" == [$self -photo_parent] && "" != [$self srcImg]} {
   puts NORM
   $self->draw.image $win [$self->create.tiled.image $win [$self srcImg]]
  } else {
   puts ABNORM
   $self->draw.image $win [$self->create.parent.image $win [$self -photo_parent]]
  }

  foreach c [winfo children $win] {
   $self->send.configure.to.children $c
  }
 }

 sd.new.method event.destroy for $obj takes {win dWin} {
  if {$win eq $dWin} {
   sd.destroy.object $self
  }
 }

 sd.new.method instance.handler for $obj takes {win instName args} {
  #puts INST:$args
  set subCmd [lindex $args 0]
  set argsLen [llength $args]
  if {"image" eq $subCmd} {
   if {2 == $argsLen} {
    $self srcImg [lindex $args 1] 
    $self->draw.image $win [$self->create.tiled.image $win [$self srcImg]]
   } elseif {1 == $argsLen} {
    return [$self img]
   } else {
    return -code error "image ?newImage?"
   } 
  } else {
   return [uplevel [linsert $args 0 $instName]]
  }
 }

 #We need to update all children that may depend on our image.
 sd.new.method send.configure.to.children for $obj takes w {
  event generate $w <Configure> -x [winfo x $w] -y [winfo y $w] \
   -width [winfo width $w] -height [winfo height $w] 
  foreach c [winfo children $w] {
   $self->send.configure.to.children $c
  } 
 }

 return $obj
}


proc PhotoFrame {win args} {
 canvas $win -width 100 -height 100 -bd 0 -highlightthickness 0

 set ar(-photo_parent) {}
 array set ar $args

 set obj [sd.new.object]
 $obj srcImg {}
 $obj img {}
 $obj id {} 

 foreach {key mem} [array get ar] {
  $obj $key $mem
 }
 PhotoMisc.methods $obj
 PhotoFrame.methods $obj 

 bind PhotoFrame$win <Configure> [list $obj->event.configure %W %w %h]
 set instName [sd.get.unique.command.name]
 rename $win $instName
 interp alias {} $win {} $obj->instance.handler $win $instName
 bindtags $win [list $win PhotoFrame$win [winfo toplevel $win] all]  

 return $win
}
