
proc pargs args {
 foreach vPtr $args {
  upvar $vPtr v 
  puts -nonewline "$vPtr:$v "
 }
 puts ""
}

proc PhotoLabel.methods obj {
 sd.new.method calculate.size.for.text for $obj takes {} {
  set w [font measure [$self -font] [$self -text]]
  array set fm [font metrics [$self -font]]
  set h [expr {$fm(-linespace) + 2}]
  return [list $w $h]
 }

 sd.new.method create.background.image for $obj takes {win w h} {
  set pWin [$self -photo_parent]
  set x [$self->calculate.offset.to.window $win $pWin x]
  set y [$self->calculate.offset.to.window $win $pWin y]
  set bgImg [image create photo -width $w -height $h]
  set pImg [[$self -photo_parent] image]

  set x2 [expr {$x + $w}]
  set y2 [expr {$y + $h}]

  $self->make.region.fit.image x y x2 y2 $pImg 

  $bgImg copy [[$self -photo_parent] image] -from $x $y $x2 $y2     
  return $bgImg
 }

 sd.new.method draw.background for $obj takes {win img} {
  if {"" != [$self bgImg]} {
   image delete [$self bgImg]
  }
  $win.c delete bgImg

  set x [expr {[image width $img] / 2}]
  set y [expr {[image height $img] / 2}]
  $win.c create image $x $y -image $img -tags bgImg
  $self bgImg $img
 }

 sd.new.method draw.text for $obj takes {win x y txt} {
  $win.c delete txt
  $win.c create text $x $y -font [$self -font] -text $txt -fill [$self -fg] -tags txt
  $win.c raise txt
 }

 sd.new.method event.configure for $obj takes {win w h} {
  $self->draw.background $win [$self->create.background.image $win $w $h]
  set x [expr {$w / 2}]
  set y [expr {$h / 2}]
  $self->draw.text $win $x $y [$self -text]
  $win.c raise text
 }

 sd.new.method event.destroy for $obj takes {dWin} {
  if {[string equal $dWin [$self win]]} {
   font delete [$self -font]
   sd.destroy.object $self
  }
 }
 
 sd.new.method instance.handler for $obj takes {win instName args} {
  #puts INST:$args
  set cmd [linsert $args 0 $instName]
  uplevel $cmd
 }
 return $obj
}

proc PhotoLabel {win args} {
 option add PhotoLabel.foreground black widgetDefault

 frame $win -class PhotoLabel -bd 0 -highlightthickness 0
 canvas $win.c -bd 0 -highlightthickness 0 -width 1 -height 1
 grid $win.c -column 0 -row 0
 grid columnconfigure $win 0 -weight 100
 grid rowconfigure $win 0 -weight 100

 set ar(-fg) [option get $win foreground PhotoLabel]
 set ar(-font) [font create -family Helvetica -size 12]
 set ar(-text) {}
 set ar(-photo_parent) {}
 array set ar $args

 set obj [sd.new.object]
 PhotoMisc.methods $obj
 PhotoLabel.methods $obj

 foreach {key data} [array get ar] {
  $obj $key $data
 }

 $obj win $win
 $obj bgImg {}

 bind PhotoLabel$win <Configure> [list $obj->event.configure %W %w %h]
 bind PhotoLabel$win <Destroy> [list $obj->event.destroy %W]

 foreach {w h} [$obj->calculate.size.for.text] {}
 $win.c config -width $w -height $h
 $win config -width $w -height $h

 set instName [sd.get.unique.command.name]
 rename $win $instName
 interp alias {} $win {} $obj->instance.handler $win $instName
 bindtags $win [list $win PhotoLabel$win [winfo toplevel $win] all]

 return $win
}
